/*
 * AT51Programmer - 	Programmer for Atmel 8051 devices, Parallel Flash and 
 * 			E2Prom memories.
 *
 * 51uni.c - 		main firmware file
 *
 * Copyright (C) 2003, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <reg52.h>
#include <stdio.h>
#include "hypAT89C52.h"
#include "cmddef.h"
#include "pinbinding.h"
#include "version.h"
#define _no                     0
#define _yes                    1

/* Serial port routines                   */
/* Receive single character with blocking */
void hypgetchar(unsigned char *rch) {
    while (!RI);
    *rch = SBUF;
    RI = 0;
}
/* Receive single character non-blocking  */
/* on success returns: 	0xFF              */
/* on error returns: 	0x00              */
unsigned char hypgetcharnoblock(unsigned char *rch) {
    if (RI) { 
	*rch = SBUF; 
	RI = 0; 
	return 0xFF; 
    }
    return 0x00;
}
/* Receive single character non-blocking  */
/* on success returns: 	(bit) 1           */
/* on error returns: 	(bit) 0           */
bit hypgetcharnoblockb(unsigned char *rch) {
    if (RI) { 
	*rch = SBUF; 
	RI = 0; 
	return 1; 
    }
    return 0;
}

/* Send single character with blocking    */
void hypputchar(unsigned char sch) {
    while (!TI);
    TI = 0;
    SBUF = sch;
}
/* Send single character non-blocking     */
/* on success returns: 	0xFF              */
/* on error returns: 	0x00              */
unsigned char hypputcharnoblock(unsigned char rch) {
    if (TI) { 
	TI = 0; 
	SBUF = rch; 
	return 0xFF; 
    }
    return 0x00;
}

/* Send single character non-blocking     */
/* on success returns: 	(bit) 1           */
/* on error returns: 	(bit) 0           */
bit hypputcharnoblockb(unsigned char rch) {
    if (TI) { 
	TI = 0; 
	SBUF = rch; 
	return 1; 
    }
    return 0;
}

void main() {
    /* serial port variables */
    unsigned char xscmd, xsdata, xsdummy;
    bit fdatareceived;
    
    /* active serial communication */
    bit factivecomm;

    /* setting serial port... to 57600 baud 8:N:1			*/
    SCON  = 0x50;
    TMOD |= 0x20;
    TH1   = 0xFF;
    TR1   = 1;
    TI    = 1;
    PCON  = 0x80;
    
    /* initializations */
    fdatareceived = _no;
    factivecomm = _no;
    
    if ( _P_DEVICE_SELECT == 0 ) { /* MASTER device */
	while (1) {
	    fdatareceived = hypgetcharnoblockb(&xscmd);
	    if (fdatareceived == _yes) {
		if (factivecomm == _no) {
		    switch (xscmd) { 
		    	case _OC_MASTER_DUMMY:
			    hypputchar(_D_STATUS_DUMMY_OK);
			    break;
			case _OC_ENABLE_COMM_REQ:
			    hypgetchar(&xsdata);
			    if (xsdata == _D_ENABLE_COMM_REQ_D0) {
				hypgetchar(&xsdata);
				if (xsdata == _D_ENABLE_COMM_REQ_D1) {
				    hypgetchar(&xsdata);
				    if (xsdata == _D_ENABLE_COMM_REQ_D2) {
					hypputchar(_D_ENABLE_COMM_ANS_D0);
					hypputchar(_D_ENABLE_COMM_ANS_D1);
					hypputchar(_D_ENABLE_COMM_ANS_D2);
					factivecomm = _yes;
				    }
				}
			    }
	    		    break;
		    } /* end of swicth */
		} /* end of factivecomm = _no */
		if (factivecomm == _yes) {
		    switch (xscmd) {
			case _OC_MASTER_SET_P0_0:  
			    P0_0 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_1:
			    P0_1 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_2:
			    P0_2 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_3:
			    P0_3 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_4:
			    P0_4 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_5:
			    P0_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_6:
			    P0_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P0_7:
			    P0_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_0:
			    P0_0 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_1:
			    P0_1 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_2:
			    P0_2 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_3:
			    P0_3 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_4:
			    P0_4 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_5:
			    P0_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_6:
			    P0_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P0_7:
			    P0_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_GET_P0_0:
			    if (P0_0) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_1:
			    if (P0_1) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_2:
			    if (P0_2) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_3:
			    if (P0_3) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_4:
			    if (P0_4) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_5:
			    if (P0_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_6:
			    if (P0_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P0_7:
			    if (P0_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_SEND_P0:
			    hypgetchar(&xsdata);
			    P0 = xsdata;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SEND_P0:
			    hypgetchar(&xsdummy);
			    break;
			case _OC_MASTER_RECV_P0:
			    hypputchar(P0);
			    break;
			case _OC_MASTER_SET_P1_0:
			    P1_0 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P1_1:
			    P1_1 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P1_2:
			    P1_2 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P1_3:
			    P1_3 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
		    	case _OC_MASTER_SET_P1_4:
			    P1_4 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P1_5:
			    P1_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P1_6:
			    P1_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P1_7:
			    P1_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_0:
			    P1_0 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_1:
			    P1_1 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_2:
			    P1_2 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_3:
			    P1_3 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_4:
			    P1_4 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_5:
			    P1_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_6:
			    P1_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P1_7:
			    P1_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_GET_P1_0:
			    if (P1_0) 
				hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_1:
			    if (P1_1) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_2:
			    if (P1_2) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_3:
			    if (P1_3) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_4:
			    if (P1_4) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_5:
			    if (P1_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_6:
			    if (P1_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P1_7:
			    if (P1_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_SEND_P1:
			    hypgetchar(&xsdata);
			    P1 = xsdata;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SEND_P1:
			    hypgetchar(&xsdummy);
			    break;
			case _OC_MASTER_RECV_P1:
			    hypputchar(P1);
			    break;
			case _OC_MASTER_SET_P2_0:  
			    P2_0 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_1:
			    P2_1 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_2:
			    P2_2 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_3:
			    P2_3 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_4:
			    P2_4 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_5:
			    P2_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_6:
			    P2_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P2_7:
			    P2_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_0:
			    P2_0 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_1:
			    P2_1 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_2:
			    P2_2 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_3:
			    P2_3 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_4:
			    P2_4 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_5:
			    P2_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_6:
			    P2_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P2_7:
			    P2_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_GET_P2_0:
			    if (P2_0) 
				hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_1:
			    if (P2_1) 
				hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_2:
			    if (P2_2) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_3:
			    if (P2_3) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_4:
			    if (P2_4) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_5:
			    if (P2_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_6:
			    if (P2_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P2_7:
			    if (P2_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_SEND_P2:
			    hypgetchar(&xsdata);
			    P2 = xsdata;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SEND_P2:
			    hypgetchar(&xsdummy);
			    break;
			case _OC_MASTER_RECV_P2:
			    hypputchar(P2);
			    break;
			case _OC_MASTER_SET_P3_5:
			    P3_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P3_6:
			    P3_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SET_P3_7:
			    P3_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P3_5:
			    P3_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P3_6:
			    P3_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_CLR_P3_7:
			    P3_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_GET_P3_5:
			    if (P3_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P3_6:
			    if (P3_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_GET_P3_7:
			    if (P3_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_MASTER_DUMMY:
			    hypputchar(_D_STATUS_DUMMY_OK);
			    break;
			case _OC_DISABLE_COMM_REQ:
			    hypgetchar(&xsdata);
			    if (xsdata == _D_DISABLE_COMM_REQ_D0) {
				hypgetchar(&xsdata);
				if (xsdata == _D_DISABLE_COMM_REQ_D1) {
				    hypgetchar(&xsdata);
				    if (xsdata == _D_DISABLE_COMM_REQ_D2) {
					hypputchar(_D_DISABLE_COMM_ANS_D0);
					hypputchar(_D_DISABLE_COMM_ANS_D1);
					hypputchar(_D_DISABLE_COMM_ANS_D2);
					factivecomm = _no;
				    }
				}
			    }
	    		    break;
		    } /* end of switch */
		} /* end of factivecomm = _yes */
	    } /* end of fdatareceived = _yes (valid data byte received!) */
	} /* end of infinite cycle when device is Master */
    } /* end of if device is Master */
    if ( _P_DEVICE_SELECT == 1 ) { /* SLAVE device */
	while (1) {
	    fdatareceived = hypgetcharnoblockb(&xscmd);
	    if (fdatareceived == _yes) {
		if (factivecomm == _no) {
		    switch (xscmd) { 
		    	case _OC_SLAVE_DUMMY:
			    hypputchar(_D_STATUS_DUMMY_OK);
			    break;
			case _OC_ENABLE_COMM_REQ:
			    hypgetchar(&xsdata);
			    if (xsdata == _D_ENABLE_COMM_REQ_D0) {
				hypgetchar(&xsdata);
				if (xsdata == _D_ENABLE_COMM_REQ_D1) {
				    hypgetchar(&xsdata);
				    if (xsdata == _D_ENABLE_COMM_REQ_D2) {
					hypputchar(_D_ENABLE_COMM_ANS_D0);
					hypputchar(_D_ENABLE_COMM_ANS_D1);
					hypputchar(_D_ENABLE_COMM_ANS_D2);
					factivecomm = _yes;
				    }
				}
			    }
	    		    break;
		    } /* end of swicth */
		} /* end of factivecomm = _no */
		if (factivecomm == _yes) {
		    switch (xscmd) {
			case _OC_SLAVE_SET_P0_0:  
			    P0_0 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_1:
			    P0_1 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_2:
			    P0_2 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_3:
			    P0_3 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_4:
			    P0_4 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_5:
			    P0_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_6:
			    P0_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P0_7:
			    P0_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_0:
			    P0_0 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_1:
			    P0_1 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_2:
			    P0_2 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_3:
			    P0_3 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_4:
			    P0_4 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_5:
			    P0_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_6:
			    P0_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P0_7:
			    P0_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_GET_P0_0:
			    if (P0_0) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_1:
			    if (P0_1) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_2:
			    if (P0_2) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_3:
			    if (P0_3) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_4:
			    if (P0_4) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_5:
			    if (P0_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_6:
			    if (P0_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P0_7:
			    if (P0_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_SEND_P0:
			    hypgetchar(&xsdata);
			    P0 = xsdata;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SEND_P0:
			    hypgetchar(&xsdummy);
			    break;
			case _OC_SLAVE_RECV_P0:
			    hypputchar(P0);
			    break;
			case _OC_SLAVE_SET_P1_0:
			    P1_0 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P1_1:
			    P1_1 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P1_2:
			    P1_2 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P1_3:
			    P1_3 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
		    	case _OC_SLAVE_SET_P1_4:
			    P1_4 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P1_5:
			    P1_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P1_6:
			    P1_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P1_7:
			    P1_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_0:
			    P1_0 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_1:
			    P1_1 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_2:
			    P1_2 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_3:
			    P1_3 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_4:
			    P1_4 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_5:
			    P1_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_6:
			    P1_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P1_7:
			    P1_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_GET_P1_0:
			    if (P1_0) 
				hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_1:
			    if (P1_1) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_2:
			    if (P1_2) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_3:
			    if (P1_3) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_4:
			    if (P1_4) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_5:
			    if (P1_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_6:
			    if (P1_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P1_7:
			    if (P1_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_SEND_P1:
			    hypgetchar(&xsdata);
			    P1 = xsdata;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SEND_P1:
			    hypgetchar(&xsdummy);
			    break;
			case _OC_SLAVE_RECV_P1:
			    hypputchar(P1);
			    break;
			case _OC_SLAVE_SET_P2_0:  
			    P2_0 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_1:
			    P2_1 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_2:
			    P2_2 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_3:
			    P2_3 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_4:
			    P2_4 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_5:
			    P2_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_6:
			    P2_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P2_7:
			    P2_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_0:
			    P2_0 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_1:
			    P2_1 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_2:
			    P2_2 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_3:
			    P2_3 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_4:
			    P2_4 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_5:
			    P2_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_6:
			    P2_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P2_7:
			    P2_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_GET_P2_0:
			    if (P2_0) 
				hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_1:
			    if (P2_1) 
				hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_2:
			    if (P2_2) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_3:
			    if (P2_3) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
				hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_4:
			    if (P2_4) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_5:
			    if (P2_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_6:
			    if (P2_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P2_7:
			    if (P2_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_SEND_P2:
			    hypgetchar(&xsdata);
			    P2 = xsdata;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_MASTER_SEND_P2:
			    hypgetchar(&xsdummy);
			    break;
			case _OC_SLAVE_RECV_P2:
			    hypputchar(P2);
			    break;
			case _OC_SLAVE_SET_P3_5:
			    P3_5 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P3_6:
			    P3_6 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_SET_P3_7:
			    P3_7 = 1;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P3_5:
			    P3_5 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P3_6:
			    P3_6 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_CLR_P3_7:
			    P3_7 = 0;
			    hypputchar(_D_STATUS_OC_ACCEPTED);
			    break;
			case _OC_SLAVE_GET_P3_5:
			    if (P3_5) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P3_6:
			    if (P3_6) 
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_GET_P3_7:
			    if (P3_7)
			        hypputchar(_D_STATUS_SINGLE_PORT_SET);
			    else
			        hypputchar(_D_STATUS_SINGLE_PORT_CLR);
			    break;
			case _OC_SLAVE_DUMMY:
			    hypputchar(_D_STATUS_DUMMY_OK);
			    break;
			case _OC_DISABLE_COMM_REQ:
			    hypgetchar(&xsdata);
			    if (xsdata == _D_DISABLE_COMM_REQ_D0) {
				hypgetchar(&xsdata);
				if (xsdata == _D_DISABLE_COMM_REQ_D1) {
				    hypgetchar(&xsdata);
				    if (xsdata == _D_DISABLE_COMM_REQ_D2) {
					hypputchar(_D_DISABLE_COMM_ANS_D0);
					hypputchar(_D_DISABLE_COMM_ANS_D1);
					hypputchar(_D_DISABLE_COMM_ANS_D2);
					factivecomm = _no;
				    }
				}
			    }
	    		    break;
		    } /* end of switch */
		} /* end of factivecomm = _yes */
	    } /* end of fdatareceived = _yes (valid data byte received!) */
	} /* end of infinite cycle when device is Slave */
    } /* end of if device is Slave */
} /* end of main() */
