/*
 * AT51Programmer - 	Programmer for Atmel 8051 devices, Parallel Flash and 
 * 			E2Prom memories.
 *
 * cmddef.h - 		Cmd opcode definitions
 *
 * Copyright (C) 2003, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#ifndef _AT51PROG_CMDDEF_H
#define _AT51PROG_CMDDEF_H

/* MASTER device opcode definitions */

/* Port 0 related definitions */
#define _OC_MASTER_SET_P0_0			0x00
#define _OC_MASTER_SET_P0_1			0x01
#define _OC_MASTER_SET_P0_2			0x02
#define _OC_MASTER_SET_P0_3			0x03
#define _OC_MASTER_SET_P0_4			0x04
#define _OC_MASTER_SET_P0_5			0x05
#define _OC_MASTER_SET_P0_6			0x06
#define _OC_MASTER_SET_P0_7			0x07

#define _OC_MASTER_CLR_P0_0			0x08
#define _OC_MASTER_CLR_P0_1			0x09
#define _OC_MASTER_CLR_P0_2			0x0A
#define _OC_MASTER_CLR_P0_3			0x0B
#define _OC_MASTER_CLR_P0_4			0x0C
#define _OC_MASTER_CLR_P0_5			0x0D
#define _OC_MASTER_CLR_P0_6			0x0E
#define _OC_MASTER_CLR_P0_7			0x0F

#define _OC_MASTER_GET_P0_0			0x40
#define _OC_MASTER_GET_P0_1			0x41
#define _OC_MASTER_GET_P0_2			0x42
#define _OC_MASTER_GET_P0_3			0x43
#define _OC_MASTER_GET_P0_4			0x44
#define _OC_MASTER_GET_P0_5			0x45
#define _OC_MASTER_GET_P0_6			0x46
#define _OC_MASTER_GET_P0_7			0x47

#define _OC_MASTER_SEND_P0			0x48
#define _OC_MASTER_RECV_P0			0x49

/* Port 1 related definitions */
#define _OC_MASTER_SET_P1_0			0x10
#define _OC_MASTER_SET_P1_1			0x11
#define _OC_MASTER_SET_P1_2			0x12
#define _OC_MASTER_SET_P1_3			0x13
#define _OC_MASTER_SET_P1_4			0x14
#define _OC_MASTER_SET_P1_5			0x15
#define _OC_MASTER_SET_P1_6			0x16
#define _OC_MASTER_SET_P1_7			0x17

#define _OC_MASTER_CLR_P1_0			0x18
#define _OC_MASTER_CLR_P1_1			0x19
#define _OC_MASTER_CLR_P1_2			0x1A
#define _OC_MASTER_CLR_P1_3			0x1B
#define _OC_MASTER_CLR_P1_4			0x1C
#define _OC_MASTER_CLR_P1_5			0x1D
#define _OC_MASTER_CLR_P1_6			0x1E
#define _OC_MASTER_CLR_P1_7			0x1F

#define _OC_MASTER_GET_P1_0			0x50
#define _OC_MASTER_GET_P1_1			0x51
#define _OC_MASTER_GET_P1_2			0x52
#define _OC_MASTER_GET_P1_3			0x53
#define _OC_MASTER_GET_P1_4			0x54
#define _OC_MASTER_GET_P1_5			0x55
#define _OC_MASTER_GET_P1_6			0x56
#define _OC_MASTER_GET_P1_7			0x57

#define _OC_MASTER_SEND_P1			0x58
#define _OC_MASTER_RECV_P1			0x59

/* Port 2 related definitions */
#define _OC_MASTER_SET_P2_0			0x20
#define _OC_MASTER_SET_P2_1			0x21
#define _OC_MASTER_SET_P2_2			0x22
#define _OC_MASTER_SET_P2_3			0x23
#define _OC_MASTER_SET_P2_4			0x24
#define _OC_MASTER_SET_P2_5			0x25
#define _OC_MASTER_SET_P2_6			0x26
#define _OC_MASTER_SET_P2_7			0x27

#define _OC_MASTER_CLR_P2_0			0x28
#define _OC_MASTER_CLR_P2_1			0x29
#define _OC_MASTER_CLR_P2_2			0x2A
#define _OC_MASTER_CLR_P2_3			0x2B
#define _OC_MASTER_CLR_P2_4			0x2C
#define _OC_MASTER_CLR_P2_5			0x2D
#define _OC_MASTER_CLR_P2_6			0x2E
#define _OC_MASTER_CLR_P2_7			0x2F

#define _OC_MASTER_GET_P2_0			0x60
#define _OC_MASTER_GET_P2_1			0x61
#define _OC_MASTER_GET_P2_2			0x62
#define _OC_MASTER_GET_P2_3			0x63
#define _OC_MASTER_GET_P2_4			0x64
#define _OC_MASTER_GET_P2_5			0x65
#define _OC_MASTER_GET_P2_6			0x66
#define _OC_MASTER_GET_P2_7			0x67

#define _OC_MASTER_SEND_P2			0x68
#define _OC_MASTER_RECV_P2			0x69

/* Port 3 related definitions */
/* Note that port 3 is not full, 4 pins are used for the RS-232 communication 
   and 1 for device select (Master/Slave) */
#define _OC_MASTER_SET_P3_5			0x35
#define _OC_MASTER_SET_P3_6			0x36
#define _OC_MASTER_SET_P3_7			0x37

#define _OC_MASTER_CLR_P3_5			0x3D
#define _OC_MASTER_CLR_P3_6			0x3E
#define _OC_MASTER_CLR_P3_7			0x3F

#define _OC_MASTER_GET_P3_5			0x75
#define _OC_MASTER_GET_P3_6			0x76
#define _OC_MASTER_GET_P3_7			0x77

#define _OC_MASTER_DUMMY			0x7D

/* SLAVE device opcode definitions */

/* Port 0 related definitions */
#define _OC_SLAVE_SET_P0_0			0x80
#define _OC_SLAVE_SET_P0_1			0x81
#define _OC_SLAVE_SET_P0_2			0x82
#define _OC_SLAVE_SET_P0_3			0x83
#define _OC_SLAVE_SET_P0_4			0x84
#define _OC_SLAVE_SET_P0_5			0x85
#define _OC_SLAVE_SET_P0_6			0x86
#define _OC_SLAVE_SET_P0_7			0x87

#define _OC_SLAVE_CLR_P0_0			0x88
#define _OC_SLAVE_CLR_P0_1			0x89
#define _OC_SLAVE_CLR_P0_2			0x8A
#define _OC_SLAVE_CLR_P0_3			0x8B
#define _OC_SLAVE_CLR_P0_4			0x8C
#define _OC_SLAVE_CLR_P0_5			0x8D
#define _OC_SLAVE_CLR_P0_6			0x8E
#define _OC_SLAVE_CLR_P0_7			0x8F

#define _OC_SLAVE_GET_P0_0			0xC0
#define _OC_SLAVE_GET_P0_1			0xC1
#define _OC_SLAVE_GET_P0_2			0xC2
#define _OC_SLAVE_GET_P0_3			0xC3
#define _OC_SLAVE_GET_P0_4			0xC4
#define _OC_SLAVE_GET_P0_5			0xC5
#define _OC_SLAVE_GET_P0_6			0xC6
#define _OC_SLAVE_GET_P0_7			0xC7

#define _OC_SLAVE_SEND_P0			0xC8
#define _OC_SLAVE_RECV_P0			0xC9

/* Port 1 related definitions */
#define _OC_SLAVE_SET_P1_0			0x90
#define _OC_SLAVE_SET_P1_1			0x91
#define _OC_SLAVE_SET_P1_2			0x92
#define _OC_SLAVE_SET_P1_3			0x93
#define _OC_SLAVE_SET_P1_4			0x94
#define _OC_SLAVE_SET_P1_5			0x95
#define _OC_SLAVE_SET_P1_6			0x96
#define _OC_SLAVE_SET_P1_7			0x97

#define _OC_SLAVE_CLR_P1_0			0x98
#define _OC_SLAVE_CLR_P1_1			0x99
#define _OC_SLAVE_CLR_P1_2			0x9A
#define _OC_SLAVE_CLR_P1_3			0x9B
#define _OC_SLAVE_CLR_P1_4			0x9C
#define _OC_SLAVE_CLR_P1_5			0x9D
#define _OC_SLAVE_CLR_P1_6			0x9E
#define _OC_SLAVE_CLR_P1_7			0x9F

#define _OC_SLAVE_GET_P1_0			0xD0
#define _OC_SLAVE_GET_P1_1			0xD1
#define _OC_SLAVE_GET_P1_2			0xD2
#define _OC_SLAVE_GET_P1_3			0xD3
#define _OC_SLAVE_GET_P1_4			0xD4
#define _OC_SLAVE_GET_P1_5			0xD5
#define _OC_SLAVE_GET_P1_6			0xD6
#define _OC_SLAVE_GET_P1_7			0xD7

#define _OC_SLAVE_SEND_P1			0xD8
#define _OC_SLAVE_RECV_P1			0xD9

/* Port 2 related definitions */
#define _OC_SLAVE_SET_P2_0			0xA0
#define _OC_SLAVE_SET_P2_1			0xA1
#define _OC_SLAVE_SET_P2_2			0xA2
#define _OC_SLAVE_SET_P2_3			0xA3
#define _OC_SLAVE_SET_P2_4			0xA4
#define _OC_SLAVE_SET_P2_5			0xA5
#define _OC_SLAVE_SET_P2_6			0xA6
#define _OC_SLAVE_SET_P2_7			0xA7

#define _OC_SLAVE_CLR_P2_0			0xA8
#define _OC_SLAVE_CLR_P2_1			0xA9
#define _OC_SLAVE_CLR_P2_2			0xAA
#define _OC_SLAVE_CLR_P2_3			0xAB
#define _OC_SLAVE_CLR_P2_4			0xAC
#define _OC_SLAVE_CLR_P2_5			0xAD
#define _OC_SLAVE_CLR_P2_6			0xAE
#define _OC_SLAVE_CLR_P2_7			0xAF

#define _OC_SLAVE_GET_P2_0			0xE0
#define _OC_SLAVE_GET_P2_1			0xE1
#define _OC_SLAVE_GET_P2_2			0xE2
#define _OC_SLAVE_GET_P2_3			0xE3
#define _OC_SLAVE_GET_P2_4			0xE4
#define _OC_SLAVE_GET_P2_5			0xE5
#define _OC_SLAVE_GET_P2_6			0xE6
#define _OC_SLAVE_GET_P2_7			0xE7

#define _OC_SLAVE_SEND_P2			0xE8
#define _OC_SLAVE_RECV_P2			0xE9

/* Port 3 related definitions */
/* Note that port 3 is not full, 4 pins are used for the RS-232 communication 
   and 1 for device select (Master/Slave) */
#define _OC_SLAVE_SET_P3_5			0xB5
#define _OC_SLAVE_SET_P3_6			0xB6
#define _OC_SLAVE_SET_P3_7			0xB7

#define _OC_SLAVE_CLR_P3_5			0xBD
#define _OC_SLAVE_CLR_P3_6			0xBE
#define _OC_SLAVE_CLR_P3_7			0xBF

#define _OC_SLAVE_GET_P3_5			0xF5
#define _OC_SLAVE_GET_P3_6			0xF6
#define _OC_SLAVE_GET_P3_7			0xF7

#define _OC_SLAVE_DUMMY				0xFD

/* MASTER / SLAVE misc opcode definitions */
#define _OC_ENABLE_COMM_REQ			0x7E
#define _OC_DISABLE_COMM_REQ			0xFE

/* Next definitions are not opcode, but unique data codes */
/* Note that the packets can contain data with the same binary code, or 
   command opcode */
#define _D_ENABLE_COMM_REQ_D0			0x68
#define _D_ENABLE_COMM_REQ_D1			0x79
#define _D_ENABLE_COMM_REQ_D2			0x70
#define _D_ENABLE_COMM_ANS_D0			0x97
#define _D_ENABLE_COMM_ANS_D1			0x86
#define _D_ENABLE_COMM_ANS_D2			0x8F
#define _D_DISABLE_COMM_REQ_D0			0x68
#define _D_DISABLE_COMM_REQ_D1			0x79
#define _D_DISABLE_COMM_REQ_D2			0x70
#define _D_DISABLE_COMM_ANS_D0			0x97
#define _D_DISABLE_COMM_ANS_D1			0x86
#define _D_DISABLE_COMM_ANS_D2			0x8F

#define _D_STATUS_OC_ACCEPTED			0x5A

#define _D_STATUS_DUMMY_OK			0xA5

#define _D_STATUS_SINGLE_PORT_SET		0xFF
#define _D_STATUS_SINGLE_PORT_CLR		0x00

#endif
